//
// HelloDrawings.h
// Copyright 2006, Konstantin Tretyakov
//
#ifndef HELLODRAWINGS_H_
#define HELLODRAWINGS_H_


void DrawSierpinski(CBitmapContext& gc, const TRect& rect);

void DrawHouse(CBitmapContext& gc, const TRect& rect);

class TMandelbrot {
public:
	TReal CenterX, CenterY, Size;
	TInt ColorMap;   // Index of the colormap to use (0, 1)
	TInt Resolution; // Number of cycles to evaluate escape time
	
	TMandelbrot();
	void Render(CBitmapContext& gc, const TRect& rect) const;
};


class TSomething {
public:
	TInt frameNo;
	
	TSomething():frameNo(0){};
	void NextFrame() { frameNo = (frameNo + 1)%255; }
	void Render(CBitmapContext& gc, const TRect& rect) const;
};


class TIFS {
public:
    // ----------------------- IFS ---------------------- //
	TInt   num_f; // Number of IFS-s in the system.
	const TReal* ifs; // Pointer to an array of N*7 values - the IFS.
			     	  // Each function is given by (a, b, c, d, e, f, p) such that
				      // x' = ax + by + c; y' = dx + ey + f;   and p is the probability
	TReal XLeft, YBottom, Height, Width; // Specifies the rectangle that will be mapped 
										 // to screen coordinates
	TInt	numiter; // Number of iterations to perform
    TRgb    color;
	TIFS();
	TIFS(TInt num_f, const TReal* ifs, TReal XLeft, TReal YBottom, TReal Width, TReal Height, TInt numiter, TRgb color);
    static TIFS GetIFS(int num);
	void Render(CBitmapContext& gc, const TRect& rect) const;
};


#endif /*HELLODRAWINGS_H_*/
