/*
============================================================================
 Name        : HelloWorldContainer.h
 Author      : Konstantin Tretyakov
 Copyright   : Copyright 2006, Konstantin Tretyakov
 Description : Declares container control for application.
============================================================================
*/

#ifndef HELLOWORLDCONTAINER_H
#define HELLOWORLDCONTAINER_H

// INCLUDES
#include <coecntrl.h>
#include "HelloDrawings.h"


// FORWARD DECLARATIONS
class CEikLabel;        // for example labels


enum TPictureType {
	EHouse = 1, ESomething, EMandelbrot, EFern1, EFern2, ESpiral, ESierpinski
};


// CLASS DECLARATION

/**
*  CHelloWorldContainer  container control class.
*
*/
class CHelloWorldContainer : public CCoeControl, MCoeControlObserver
    {
    public: // Constructors and destructor

        /**
        * EPOC default constructor.
        * @param aRect Frame rectangle for container.
        */
        void ConstructL(const TRect& aRect);

        /**
        * Destructor.
        */
        ~CHelloWorldContainer();

    public:
		void SetCurrentPic(TPictureType aCurrentPic);
		
		// dx/dy are -1/0/1 integers
		void HandleMove(TInt dx, TInt dy);
		
		void HandleOK();

		void HandleColormap(TInt index);
		
		void HandleResolution(TBool up);
		
		static TInt HandleRedrawTimer(TAny* self);
		
    public: // Functions from base classes

    private: // Functions from base classes

       /**
        * From CoeControl,SizeChanged.
        */
        void SizeChanged();

       /**
        * From CoeControl,CountComponentControls.
        */
        TInt CountComponentControls() const;

       /**
        * From CCoeControl,ComponentControl.
        */
        CCoeControl* ComponentControl(TInt aIndex) const;

       /**
        * From CCoeControl,Draw.
        */
        void Draw(const TRect& aRect) const;


	   /**
		* From MCoeControlObserver
		* Acts upon changes in the hosted control's state.
		*
		* @param	aControl	The control changing its state
		* @param	aEventType	The type of control event
		*/
        void HandleControlEventL(CCoeControl* aControl,TCoeEvent aEventType);
		
    private: //data

        CEikLabel* iLabel;          // example label
        CEikLabel* iToDoLabel;      // example label
        
        const CFont* iMyFont;

		TPictureType iCurrentPic;
		TMandelbrot  iMandelbrot;
	public:
		TSomething iSomething; // This is updated in the redraw timer

	private:
        CFbsBitmap* iOffScreenBitmap;
        CFbsBitmapDevice* iOffScreenBitmapDevice;
        CBitmapContext* iOffScreenBitmapGc;
        
        CPeriodic* iPeriodic;
    };

#endif


