// 
// Symbian Graphics "Hello World"-s
// 
// Copyright 2006, Konstantin Tretyakov
//

#include <coecntrl.h>
#include <e32math.h>
#include "HelloDrawings.h"


void DrawHouse(CBitmapContext& gc, const TRect& rect) {
	gc.SetPenStyle(CGraphicsContext::ENullPen);
    gc.SetBrushStyle( CGraphicsContext::ESolidBrush );
    TInt w = rect.Width();
    TInt h = rect.Height();

	TInt horizon = h*2/3;
    gc.SetBrushColor(KRgbWhite);
    gc.DrawRect(TRect(0, horizon+1, w, h));
    
    gc.SetBrushColor(KRgbBlue);
    gc.DrawRect(TRect(0, 0, w, horizon));
    
    gc.SetBrushColor(KRgbRed);
    TInt houseWidth = w*2/7;
    TInt houseHeight = w/5;
    TInt houseLeft = w/5;
    gc.DrawRect(TRect(houseLeft, horizon - houseHeight, houseLeft + houseWidth, horizon));
 
	TPoint roof[3] = { TPoint(houseLeft - 15, horizon - houseHeight),
					   TPoint(houseLeft + houseWidth + 15, horizon - houseHeight),
					   TPoint(houseLeft + houseWidth/2, horizon - houseHeight - houseHeight*2/3)};					   
 	gc.DrawPolygon(roof, 3);
 	
 	gc.SetBrushColor(KRgbYellow);
 	TInt sunR = w/15;
 	TInt sunX = w*3/4 + sunR;
 	TInt sunY  = sunR*2;
 	
 	gc.DrawEllipse(TRect(sunX - sunR, sunY - sunR, sunX + sunR, sunY + sunR));
 
 	gc.SetPenColor(KRgbYellow);
 	gc.SetPenStyle(CGraphicsContext::ESolidPen);
	for (TInt i = 0; i < 25; i++) {
		TReal ang = (TReal)i*3.1415*2/25;
		TReal sinang, cosang;
		Math::Sin(sinang, ang);
		Math::Cos(cosang, ang);
		TInt fromX = (TInt)((sunR+2)*cosang);
		TInt fromY = (TInt)((sunR+2)*sinang);
		TInt toX   = (TInt)((sunR+32)*cosang);
		TInt toY   = (TInt)((sunR+32)*sinang);
		gc.DrawLine(TPoint(fromX + sunX, fromY + sunY), TPoint(toX + sunX, toY + sunY));
	}
	
	
	_LIT(KMyText, "Maja");
    gc.DrawText(KMyText, TPoint(houseLeft + 5, horizon - 10));
}



// ----------------------- Something --------------------------------- //

void TSomething::Render(CBitmapContext& gc, const TRect& rect) const {
	gc.SetPenStyle(CGraphicsContext::ESolidPen);
    gc.SetBrushColor(KRgbWhite);
    gc.SetBrushStyle(CGraphicsContext::ESolidBrush);
    gc.DrawRect(rect);
    
   	for (int x = 0; x < rect.Width(); x++)
   		for (int y = 0; y < rect.Height(); y++) {
   			int r = (x + frameNo) % 255;
   			int g = (y + frameNo) % 255;
   			int b = 0;
   			gc.SetPenColor(TRgb(r, g, b));
   			gc.Plot(TPoint(x,y));
   		}
}


// ----------------------- IFS Fractals --------------------------------- //

TIFS::TIFS() {};

TIFS TIFS::GetIFS(int num) {
    // To avoid global data (Symbian does not like it)
    static const TReal sierpinski_ifs[3*7] = {
            0.5,   0,   0,   0, 0.5, 0.5, 0.33,
            0.5,   0, 0.5,   0, 0.5, 0.5, 0.33,
            0.5,   0,0.25,   0, 0.5,  0,  0.34
    };
    
    
    static const TReal fern1_ifs[4*7] = {
        0.0, 0.0, 0.0, 0.0, 0.16, 0.0, 0.1,
        0.2, -0.26, 0.0, 0.23, 0.22, 1.6, 0.08,
        -0.15, 0.28, 0.0, 0.26, 0.24, 0.44, 0.08,
        0.75, 0.04, 0.0, -0.04, 0.85, 1.6, 0.74
    };
    
    static const TReal fern2_ifs[4*7] = {
        0.0, 0.0, 0.0, 0.0, 0.16, 0.0, 0.01,
        0.2, -0.26, 0.0, 0.23, 0.22, 1.6, 0.07,
        -0.15, 0.28, 0.0, 0.26, 0.24, 0.44, 0.07,
        0.85, 0.04, 0.0, -0.04, 0.85, 1.6, 0.85
    };
    
    static const TReal spiral_ifs[3*7] = {
        0.787879, -0.424242, 1.758647, 0.242424, 0.859848, 1.408065, 0.90,
        -0.121212,  0.257576, -6.721654, 1.377236, 0.151515, 0.053030, 0.05,
        0.181818, -0.136364, 6.086107, 1.568035, 0.090909, 0.181818,  0.05
    };
    
    switch(num) {
        case 0:
            return TIFS(3, sierpinski_ifs, 0, 0, 1, 1, 3000, KRgbBlack);
        case 1:        
            return TIFS(4, fern1_ifs, -6, 11, 12, -12, 10000, TRgb(0, 100, 0));
        case 2:
            return TIFS(4, fern2_ifs, -6, 11, 12, -12, 10000, TRgb(0, 100, 0));
        case 3:
            return TIFS(3, spiral_ifs, -21, -15, 40, 40, 10000, KRgbBlack);
    }
}
    
TIFS::TIFS(TInt num_f, const TReal* ifs, TReal XLeft, TReal YBottom, TReal Width, TReal Height, TInt numiter, TRgb color):
	num_f(num_f), ifs(ifs), XLeft(XLeft), YBottom(YBottom), Height(Height), Width(Width), numiter(numiter), color(color) {};

void TIFS::Render(CBitmapContext& gc, const TRect& rect) const {
	// Clear screen
	gc.SetPenStyle(CGraphicsContext::ENullPen);
    gc.SetBrushColor(KRgbWhite);
    gc.SetBrushStyle(CGraphicsContext::ESolidBrush);
    gc.DrawRect(rect);
    
	gc.SetPenStyle(CGraphicsContext::ESolidPen);    
    gc.SetPenColor(color);
    
	TReal x = 0, y = 0, nx, ny;
	TInt cur_f = 0, test_val;
	TReal a, b, c, d, e, f;
	TInt w = rect.Width(), h = rect.Height();
	
	//TReal XLeft = -5, YBottom = 9, Width = 10, Height = -10;
	
	for (TInt i = 0; i < numiter; i++) {
		gc.Plot(TPoint((x - XLeft)*w/Width, (y - YBottom)*h/Height));
		
		// Pick a function according to the probability distribution
		test_val = Math::Random() % 100;
		for (TInt f = 0; f < num_f; f++) {
			if (ifs[7*f + 6]*100 >= test_val) {
				cur_f = f;
				break;
			}
			else test_val = test_val - (TInt)(ifs[7*f + 6]*100);
		}
		
		a = ifs[7*cur_f]; b = ifs[7*cur_f + 1]; c = ifs[7*cur_f + 2];
		d = ifs[7*cur_f + 3]; e = ifs[7*cur_f + 4]; f = ifs[7*cur_f + 5];
		
		// Apply the function
		nx = a*x + b*y + c;
		ny = d*x + e*y + f;
		x  = nx;
		y  = ny;
	}
}
	
// ----------------------- Mandelbrot --------------------------------- //


// Default fractal settings
TMandelbrot::TMandelbrot():
	CenterX(-0.7845), CenterY(0.1435), Size(0.017), Resolution(100), ColorMap(0) {
}



void TMandelbrot::Render(CBitmapContext& gc, const TRect& rect) const {
    // Symbian does not allow static data!
    const TRgb colormapJet[31] = {
    TRgb(0,0,144),
    TRgb(0,0,178),
    TRgb(0,0,212),
    TRgb(0,0,246),
    TRgb(0,25,255),
    TRgb(0,59,255),
    TRgb(0,93,255),
    TRgb(0,127,255),
    TRgb(0,161,255),
    TRgb(0,195,255),
    TRgb(0,229,255),
    TRgb(8,255,246),
    TRgb(42,255,212),
    TRgb(76,255,178),
    TRgb(110,255,144),
    TRgb(144,255,110),
    TRgb(178,255,76),
    TRgb(212,255,42),
    TRgb(246,255,8),
    TRgb(255,229,0),
    TRgb(255,195,0),
    TRgb(255,161,0),
    TRgb(255,127,0),
    TRgb(255,93,0),
    TRgb(255,59,0),
    TRgb(255,25,0),
    TRgb(246,0,0),
    TRgb(212,0,0),
    TRgb(178,0,0),
    TRgb(144,0,0),
    KRgbBlack
    };
    
    const TRgb colormapHot[31] = {
    TRgb(23,0,0),
    TRgb(46,0,0),
    TRgb(69,0,0),
    TRgb(92,0,0),
    TRgb(115,0,0),
    TRgb(139,0,0),
    TRgb(162,0,0),
    TRgb(185,0,0),
    TRgb(208,0,0),
    TRgb(231,0,0),
    TRgb(255,0,0),
    TRgb(255,23,0),
    TRgb(255,46,0),
    TRgb(255,69,0),
    TRgb(255,92,0),
    TRgb(255,115,0),
    TRgb(255,139,0),
    TRgb(255,162,0),
    TRgb(255,185,0),
    TRgb(255,208,0),
    TRgb(255,231,0),
    TRgb(255,255,0),
    TRgb(255,255,28),
    TRgb(255,255,56),
    TRgb(255,255,85),
    TRgb(255,255,113),
    TRgb(255,255,141),
    TRgb(255,255,170),
    TRgb(255,255,198),
    TRgb(255,255,226),
    TRgb(255,255,255)
    };
    
    const TRgb* colormaps[2] = {colormapHot, colormapJet};

    
	gc.SetPenStyle(CGraphicsContext::ESolidPen);
	
	TInt w = rect.Width();
	TInt h = rect.Height();

	TReal xLow = this->CenterX - this->Size/2;
	TReal yLow = this->CenterY - this->Size/2;
	
	for (TInt i = 0; i < w; i++) {
		for (TInt j = 0; j < h; j++) {
			
			// Coordinates of the pixel
			TReal x0 = (float)(i)*this->Size/w + xLow;
			TReal y0 = (float)(j)*this->Size/h + yLow;
			TReal x = x0;
			TReal y = y0;
			TReal x2 = x*x;
			TReal y2 = y*y;
			
			// Find escape time
			TInt iter = 0;
			TInt maxiter = this->Resolution;
			while ((x2 + y2 < 2*2) && (iter < maxiter)) {
				y = 2*x*y + y0;
				x = x2 - y2 + x0;
				x2 = x*x;
				y2 = y*y;
				iter++;
			}
			
			TInt escapetime = iter*30/maxiter;
			gc.SetPenColor(colormaps[this->ColorMap][escapetime]);
			gc.Plot(TPoint(i, j));
		}
	}
}


