/*
============================================================================
 Name        : CHelloWorldAppUi.cpp
 Author      : Konstantin Tretyakov
 Copyright   : Copyright 2006, Konstantin Tretyakov
 Description : CHelloWorldAppUi implementation
============================================================================
*/

// INCLUDE FILES
#include "HelloWorldAppui.h"
#include "HelloWorldContainer.h"
#include <HelloWorld.rsg>
#include "HelloWorld.hrh"

#include <avkon.hrh>
#include <aknnotewrappers.h>
#include <aknnotedialog.h>


// ================= MEMBER FUNCTIONS =======================
//
// ----------------------------------------------------------
// CHelloWorldAppUi::ConstructL()
//
// ----------------------------------------------------------
//
void CHelloWorldAppUi::ConstructL()
    {
    BaseConstructL();

    iAppContainer = new (ELeave) CHelloWorldContainer;
    iAppContainer->SetMopParent( this );
    iAppContainer->ConstructL( ClientRect() );
    AddToStackL( iAppContainer );
    }

// ----------------------------------------------------
// CHelloWorldAppUi::~CHelloWorldAppUi()
// Destructor
// Frees reserved resources
// ----------------------------------------------------
//
CHelloWorldAppUi::~CHelloWorldAppUi()
    {
    if (iAppContainer)
        {
        RemoveFromStack( iAppContainer );
        delete iAppContainer;
        }
   }

// ------------------------------------------------------------------------------
// CHelloWorldAppUi::DynInitMenuPaneL(TInt aResourceId,CEikMenuPane* aMenuPane)
//  This function is called by the EIKON framework just before it displays
//  a menu pane. Its default implementation is empty, and by overriding it,
//  the application can set the state of menu items dynamically according
//  to the state of application data.
// ------------------------------------------------------------------------------
//
void CHelloWorldAppUi::DynInitMenuPaneL(
    TInt /*aResourceId*/,CEikMenuPane* /*aMenuPane*/)
    {
    }

// ----------------------------------------------------
// CHelloWorldAppUi::HandleKeyEventL(
//     const TKeyEvent& aKeyEvent,TEventCode /*aType*/)
// takes care of key event handling
// ----------------------------------------------------
//
TKeyResponse CHelloWorldAppUi::HandleKeyEventL(const TKeyEvent& aKeyEvent, TEventCode aType) {
	if (aType == EEventKey) {
		switch(aKeyEvent.iScanCode){
			case EStdKeyUpArrow:	iAppContainer->HandleMove( 0,-1); break;
			case EStdKeyDownArrow:	iAppContainer->HandleMove( 0, 1); break;
			case EStdKeyLeftArrow:	iAppContainer->HandleMove(-1, 0); break;
			case EStdKeyRightArrow:iAppContainer->HandleMove( 1, 0); break;
			case EStdKeyDevice3:	iAppContainer->HandleOK(); break; // OK button
			//case EStdKeyUpArrow:	iAppContainer->HandleToggleResolution(); break;
			//case EStdKeyDownArrow:	iAppContainer->HandleToggleColormap(); break;
			default: return EKeyWasNotConsumed;
	    }
	    return EKeyWasConsumed;
	}
	else return EKeyWasNotConsumed;
}

// ----------------------------------------------------
// CHelloWorldAppUi::HandleCommandL(TInt aCommand)
// takes care of command handling
// ----------------------------------------------------
//
void CHelloWorldAppUi::HandleCommandL(TInt aCommand)
    {
    switch ( aCommand )
        {
        case EAknSoftkeyBack:
        case EEikCmdExit:
        case EAknCmdExit:
        {
        	// Ask confirmation
        	CAknQueryDialog* dlg = CAknQueryDialog::NewL();
			HBufC* msgSure = CEikonEnv::Static()->AllocReadResourceLC(R_ARE_YOU_SURE);
				        	
        	dlg->SetPromptL(msgSure->Des());
        	
        	if (dlg->ExecuteLD(R_ARE_YOU_SURE_QUERY)){
				CleanupStack::PopAndDestroy(msgSure);        		
        		// Replied yes
        		Exit();
        	}
        	else {
        		CleanupStack::PopAndDestroy(msgSure);
        		// Replied no. Show a message box.
				HBufC* message = CEikonEnv::Static()->AllocReadResourceLC(R_MESSAGE_TEXT);
				
				// Show information note dialog
	            CAknInformationNote* note = new (ELeave) CAknInformationNote;
	            note->ExecuteLD(message->Des());
				CleanupStack::PopAndDestroy(message);
        	}
            break;
        }
        case ECmdShowHouse:
        	iAppContainer->SetCurrentPic(EHouse);
        	break;
       	case ECmdShowSomething:
       		iAppContainer->SetCurrentPic(ESomething);
       		break;
       	case ECmdShowMandelbrot:
       		iAppContainer->SetCurrentPic(EMandelbrot);
       		break;
       	case ECmdShowFern1:
       		iAppContainer->SetCurrentPic(EFern1);
       		break;
       	case ECmdShowFern2:
       		iAppContainer->SetCurrentPic(EFern2);
       		break;       		       		
       	case ECmdShowSierpinski:
       		iAppContainer->SetCurrentPic(ESierpinski);
       		break;
        case ECmdShowSpiral:
            iAppContainer->SetCurrentPic(ESpiral);
            break;
       	case ECmdColormapHot:
       		iAppContainer->HandleColormap(0);
       		break;
       	case ECmdColormapJet:
       		iAppContainer->HandleColormap(1);
       		break;
       	case ECmdResolutionInc:
       		iAppContainer->HandleResolution(ETrue);
       		break;
       	case ECmdResolutionDec:
       		iAppContainer->HandleResolution(EFalse);
       		break;
        default:
            break;
        }
    }




