/*
============================================================================
 Name        : HelloWorldContainer.cpp
 Author      : Konstantin Tretyakov
 Copyright   : Copyright 2006, Konstantin Tretyakov
 Description : Container control implementation
============================================================================
*/

// INCLUDE FILES
#include "HelloWorldContainer.h"
#include "bitmapmethods.h"

#include <eiklabel.h>  // for example label control


// ================= MEMBER FUNCTIONS =======================

// ---------------------------------------------------------
// CHelloWorldContainer::ConstructL(const TRect& aRect)
// EPOC two phased constructor
// ---------------------------------------------------------
//
void CHelloWorldContainer::ConstructL(const TRect& aRect) {
    CreateWindowL();

    iLabel = new (ELeave) CEikLabel;
    iLabel->SetContainerWindowL( *this );
    iLabel->SetTextL(_L(""));

    iToDoLabel = new (ELeave) CEikLabel;
    iToDoLabel->SetContainerWindowL( *this );
    iToDoLabel->SetTextL(_L(""));

	iCurrentPic = EFern2;
	
    SetRect(aRect);

    // Create the off screen bitmap and device / gc
    iOffScreenBitmap       = NBitmapMethods::
							CreateBitmapL(Rect().Size(),KColourDepth);
    iOffScreenBitmapDevice = NBitmapMethods::
							CreateBitmapDeviceL(*iOffScreenBitmap);
    iOffScreenBitmapGc     = NBitmapMethods::
							CreateGraphicsContextL(*iOffScreenBitmapDevice);

	// Setup font    
//   	_LIT(KMyFontName,"Arial");
//	TFontSpec myFontSpec(KMyFontName, 20);
//	iOffScreenBitmapDevice->GetNearestFontInTwips(iMyFont, myFontSpec);
	iMyFont = ControlEnv()->NormalFont(); // Also - TitleFont()
	iOffScreenBitmapGc->UseFont(iMyFont);
	
	// Create timer
	iPeriodic = CPeriodic::NewL(CActive::EPriorityIdle);

    ActivateL();
}

// Destructor
CHelloWorldContainer::~CHelloWorldContainer()
{
    delete iLabel;
    delete iToDoLabel;

//	iOffScreenBitmapDevice->ReleaseFont(iMyFont);
	delete iOffScreenBitmapGc;
	delete iOffScreenBitmapDevice;
	delete iOffScreenBitmap;
	delete iPeriodic;
}

// ---------------------------------------------------------
// CHelloWorldContainer::SizeChanged()
// Called by framework when the view size is changed
// ---------------------------------------------------------
//
void CHelloWorldContainer::SizeChanged()
    {
    // TODO: Add here control resize code etc.
    iLabel->SetExtent( TPoint(10,10), iLabel->MinimumSize() );
    iToDoLabel->SetExtent( TPoint(10,100), iToDoLabel->MinimumSize() );
    }

// ---------------------------------------------------------
// CHelloWorldContainer::CountComponentControls() const
// ---------------------------------------------------------
//
TInt CHelloWorldContainer::CountComponentControls() const
    {
    return 2; // return nbr of controls inside this container
    }

// ---------------------------------------------------------
// CHelloWorldContainer::ComponentControl(TInt aIndex) const
// ---------------------------------------------------------
//
CCoeControl* CHelloWorldContainer::ComponentControl(TInt aIndex) const
    {
    switch ( aIndex )
        {
        case 0:
            return iLabel;
        case 1:
            return iToDoLabel;
        default:
            return NULL;
        }
    }

// ---------------------------------------------------------
// CHelloWorldContainer::Draw(const TRect& aRect) const
// ---------------------------------------------------------
//
void CHelloWorldContainer::Draw(const TRect& /*aRect*/) const
{
    CWindowGc& gc = SystemGc();
    const TRect& rect = Rect();

    switch (iCurrentPic) {
    	case EHouse: DrawHouse(*iOffScreenBitmapGc, rect); break;
    	case ESomething: iSomething.Render(*iOffScreenBitmapGc, rect); break;
    	case EMandelbrot: iMandelbrot.Render(*iOffScreenBitmapGc, rect); break;
    	case ESierpinski: TIFS::GetIFS(0).Render(*iOffScreenBitmapGc, rect); break;
    	case EFern1: TIFS::GetIFS(1).Render(*iOffScreenBitmapGc, rect); break;
    	case EFern2: TIFS::GetIFS(2).Render(*iOffScreenBitmapGc, rect); break;
        case ESpiral: TIFS::GetIFS(3).Render(*iOffScreenBitmapGc, rect); break;        
    }
    gc.BitBlt(rect.iTl, iOffScreenBitmap);    
}

// ---------------------------------------------------------
// CHelloWorldContainer::HandleControlEventL(
//     CCoeControl* aControl,TCoeEvent aEventType)
// ---------------------------------------------------------
//
void CHelloWorldContainer::HandleControlEventL(
    CCoeControl* /*aControl*/,TCoeEvent /*aEventType*/)
    {
    // TODO: Add your control event handler code here
    }



void CHelloWorldContainer::SetCurrentPic(TPictureType aCurrentPic){
	if (aCurrentPic == ESomething) {	// Stop timer
		delete iPeriodic;
		iPeriodic = CPeriodic::NewL(CActive::EPriorityIdle);
	}
	
	iCurrentPic = aCurrentPic;
	
	if (aCurrentPic == ESomething) {	// Start timer
		iPeriodic->Start(100, 100, TCallBack(CHelloWorldContainer::HandleRedrawTimer, this));
	}
	
	DrawNow();
}


void CHelloWorldContainer::HandleMove(TInt dx, TInt dy) {
	TReal stepSize = iMandelbrot.Size/4;
	iMandelbrot.CenterX += dx*stepSize;
	iMandelbrot.CenterY += dy*stepSize;
	DrawNow();
}

void CHelloWorldContainer::HandleOK() {
	iMandelbrot.Size /= 2;
	DrawNow();
}

void CHelloWorldContainer::HandleColormap(TInt index) {
	iMandelbrot.ColorMap = index;
	DrawNow();
}

void CHelloWorldContainer::HandleResolution(TBool up) {
	if (up) iMandelbrot.Resolution = iMandelbrot.Resolution*3/2;
	else	iMandelbrot.Resolution = iMandelbrot.Resolution*2/3;
	DrawNow();
}


TInt CHelloWorldContainer::HandleRedrawTimer(TAny* self) {
	CHelloWorldContainer* instance = (CHelloWorldContainer*) self;
    instance->iSomething.NextFrame();
    instance->DrawNow();
    return 0;
}


		
